@@ -1,4 +1,5 @@
-/* -*- mode: c++; c-basic-offset:4 -*-
+/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil; -*-
+
     utils/formatting.cpp
 
     This file is part of Kleopatra, the KDE keymanager
@@ -36,6 +37,9 @@
 #include <gpgme++/key.h>
 #include <gpgme++/importresult.h>
 
+#include <QGpgME/CryptoConfig>
+#include <QGpgME/Protocol>
+
 #include <KLocalizedString>
 #include <KEmailAddress>
 
@@ -46,6 +50,11 @@
 #include <QLocale>
 #include <QIcon>
 
+#include <gpgme++/gpgmepp_version.h>
+#if GPGMEPP_VERSION > 0x10900
+# define GPGME_HAS_KEY_IS_DEVS
+#endif
+
 
 using namespace GpgME;
 using namespace Kleo;
@@ -253,19 +262,19 @@ QString format_keyusage(const Key &key)
     QStringList capabilities;
     if (key.canReallySign()) {
         if (key.isQualified()) {
-            capabilities.push_back(i18n("Signing EMails and Files (Qualified)"));
+            capabilities.push_back(i18n("Signing (Qualified)"));
         } else {
-            capabilities.push_back(i18n("Signing EMails and Files"));
+            capabilities.push_back(i18n("Signing"));
         }
     }
     if (key.canEncrypt()) {
-        capabilities.push_back(i18n("Encrypting EMails and Files"));
+        capabilities.push_back(i18n("Encryption"));
     }
     if (key.canCertify()) {
-        capabilities.push_back(i18n("Certifying other Certificates"));
+        capabilities.push_back(i18n("Certifying Identities"));
     }
     if (key.canAuthenticate()) {
-        capabilities.push_back(i18n("Authenticate against Servers"));
+        capabilities.push_back(i18n("SSH Authentication"));
     }
     return capabilities.join(QStringLiteral(", "));
 }
@@ -296,16 +305,30 @@ QString Formatting::toolTip(const Key &key, int flags)
     if (flags & Validity) {
         if (key.protocol() == OpenPGP || (key.keyListMode() & Validate))
             if (key.isRevoked()) {
-                result += make_red(i18n("This certificate has been revoked."));
+                result = make_red(i18n("Revoked"));
             } else if (key.isExpired()) {
-                result += make_red(i18n("This certificate has expired."));
+                result = make_red(i18n("Expired"));
             } else if (key.isDisabled()) {
-                result += i18n("This certificate has been disabled locally.");
+                result = i18n("Disabled");
             } else {
-                result += i18n("This certificate is currently valid.");
+                unsigned int fullyTrusted = 0;
+                for (const auto &uid: key.userIDs()) {
+                    if (uid.validity() >= UserID::Validity::Full) {
+                        fullyTrusted++;
+                    }
+                }
+                if (fullyTrusted == key.numUserIDs()) {
+                    result = i18n("All identities are certified.");
+                    const auto compliance = complianceStringForKey(key);
+                    if (!compliance.isEmpty()) {
+                        result += QStringLiteral("<br>") + compliance;
+                    }
+                } else {
+                    result = i18np("One identity is not certified.", "%1 identities are not certified.", key.numUserIDs() - fullyTrusted);
+                }
             }
         else {
-            result += i18n("The validity of this certificate cannot be checked at the moment.");
+            result = i18n("The validity cannot be checked at the moment.");
         }
     }
     if (flags == Validity) {
@@ -333,16 +356,20 @@ QString Formatting::toolTip(const Key &key, int flags)
                     result += format_row(i18n("a.k.a."), prettyUserID(*it));
                 }
     }
-    if (flags & ExpiryDates)
-        result += format_row(i18n("Validity"),
-                             subkey.neverExpires()
-                             ? i18n("from %1 until forever", time_t2string(subkey.creationTime()))
-                             : i18n("from %1 through %2", time_t2string(subkey.creationTime()), time_t2string(subkey.expirationTime())));
+    if (flags & ExpiryDates) {
+        result += format_row(i18n("Created"), time_t2string(subkey.creationTime()));
+
+        if (key.isExpired()) {
+            result += format_row(i18n("Expired"), time_t2string(subkey.expirationTime()));
+        } else if (!subkey.neverExpires()) {
+            result += format_row(i18n("Expires"), time_t2string(subkey.expirationTime()));
+        }
+    }
     if (flags & CertificateType) {
-        result += format_row(i18n("Certificate type"), format_keytype(key));
+        result += format_row(i18n("Type"), format_keytype(key));
     }
     if (flags & CertificateUsage) {
-        result += format_row(i18n("Certificate usage"), format_keyusage(key));
+        result += format_row(i18n("Usage"), format_keyusage(key));
     }
     if (flags & KeyID) {
         result += format_row(i18n("Key-ID"), QString::fromLatin1(key.shortKeyID()));
@@ -566,6 +593,9 @@ QString Formatting::validityShort(const UserID::Signature &sig)
                 return i18nc("good/valid signature", "good");
             }
         }
+#if QT_VERSION >= QT_VERSION_CHECK(5,8,0)
+        Q_FALLTHROUGH();
+#endif
     // fall through:
     case UserID::Signature::GeneralError:
         return i18n("invalid");
@@ -751,8 +781,10 @@ QString Formatting::usageString(const Subkey &sub)
 QString Formatting::summaryLine(const Key &key)
 {
     return keyToString(key) + QLatin1Char(' ') +
-           i18nc("First arg is the Key Protocol OpenPGP or S/MIME, second arg is the creation date.",
-                 "(%1 - created: %2)", displayName(key.protocol()) ,
+           i18nc("(validity, protocol, creation date)",
+                 "(%1, %2, created: %3)",
+		 Formatting::complianceStringShort(key),
+		 displayName(key.protocol()),
                  Formatting::creationDateString(key));
 }
 
@@ -790,3 +822,76 @@ QString Formatting::validity(const UserID &uid)
             return i18n("There is no indication that this certificate belongs to this recipient.");
     }
 }
+
+bool Formatting::uidsHaveFullValidity(const GpgME::Key &key)
+{
+    for (const auto &uid: key.userIDs()) {
+        if (uid.validity() < UserID::Validity::Full) {
+            return false;
+        }
+    }
+    return true;
+}
+
+QString Formatting::complianceMode()
+{
+    const QGpgME::CryptoConfig *const config = QGpgME::cryptoConfig();
+    if (!config) {
+        return QString();
+    }
+    const QGpgME::CryptoConfigEntry *const entry = config->entry(QStringLiteral("gpg"), QStringLiteral("Configuration"), QStringLiteral("compliance"));
+
+    if (!entry || entry->stringValue() == QStringLiteral("gnupg")) {
+        return QString();
+    }
+    return entry->stringValue();
+}
+
+bool Formatting::isKeyDeVs(const GpgME::Key &key)
+{
+#ifdef GPGME_HAS_KEY_IS_DEVS
+    for (const auto &sub: key.subkeys()) {
+        if (sub.isExpired() || sub.isRevoked()) {
+            // Ignore old subkeys
+            continue;
+        }
+        if (!sub.isDeVs()) {
+            return false;
+        }
+    }
+    return true;
+#else
+    return false;
+#endif
+}
+
+QString Formatting::complianceStringForKey(const GpgME::Key &key)
+{
+    // There will likely be more in the future for other institutions
+    // for now we only have DE-VS
+    if (complianceMode() == QStringLiteral("de-vs")) {
+        if (uidsHaveFullValidity(key) && isKeyDeVs(key)) {
+            return i18nc("VS-conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant with that..",
+                         "May be used for VS-compliant communication.");
+        } else {
+            return i18nc("VS-conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant to that..",
+                         "May <b>not</b> be used for VS-compliant communication.");
+        }
+    }
+    return QString();
+}
+
+QString Formatting::complianceStringShort(const GpgME::Key &key)
+{
+    if (Formatting::uidsHaveFullValidity(key)) {
+        if (complianceMode() == QStringLiteral("de-vs")
+            && Formatting::isKeyDeVs(key)) {
+            return QStringLiteral("★ ") +
+                i18nc("VS-conforming is a German standard for restricted documents for which special restrictions about algorithms apply.  The string states that a key is compliant with that.",
+                      "VS-compliant");
+        }
+        return i18nc("As in all user IDs are valid.", "certified");
+    }
+
+    return i18nc("As in not all user IDs are valid.", "not certified");
+}
